@@ -32,6 +32,7 @@ R_HIGHS_PKG_HOME=`pwd`
 HIGHS_SRC_DIR=${R_HIGHS_PKG_HOME}/inst/HiGHS
 R_HIGHS_BUILD_DIR=${HIGHS_SRC_DIR}/build
 R_HIGHS_LIB_DIR=${R_HIGHS_PKG_HOME}/src/highslib
+# R_HIGHS_BUILD_DIR=~/lib/HiGHS/build
 # R_HIGHS_LIB_DIR=~/lib/highslib
 
 mkdir -p ${R_HIGHS_BUILD_DIR}
@@ -41,7 +42,7 @@ cd ${R_HIGHS_BUILD_DIR}
 
 export CC=`"${R_HOME}/bin/R" CMD config CC`
 export CXX=`"${R_HOME}/bin/R" CMD config CXX`
-export CXX11=`"${R_HOME}/bin/R" CMD config CXX11`
+export CXX17=`"${R_HOME}/bin/R" CMD config CXX17`
 export CXXFLAGS=`"${R_HOME}/bin/R" CMD config CXXFLAGS`
 export CFLAGS=`"${R_HOME}/bin/R" CMD config CFLAGS`
 export CPPFLAGS=`"${R_HOME}/bin/R" CMD config CPPFLAGS`
@@ -55,10 +56,11 @@ echo "R_ARCH: '$R_ARCH'"
 # echo "OS_TYPE: '${OS_TYPE}'"
 echo "CC: '${CC}'"
 echo "CXX: '${CXX}'"
-echo "CXX11: '${CXX11}'"
+echo "CXX17: '${CXX17}'"
+echo "R_HIGHS_BUILD_DIR: '${R_HIGHS_BUILD_DIR}'"
+echo "R_HIGHS_LIB_DIR: '${R_HIGHS_LIB_DIR}'"
 echo ""
 
-
 # The flag CMAKE_CXX_COMPILER_WORKS signals cmake that we know
 # that the compiler works therefore cmake has not to test.
 # Documentation on this flag is very hard to find.
@@ -70,13 +72,13 @@ echo ""
 # But than it fails since prior to 4.2.0 R-win tries to compile and test for 'i386' and 'x64'
 # and fails for 'i386'.
 # if test "${OS_TYPE}" = "unix"; then
-CMAKE_OPTS="-DCMAKE_INSTALL_PREFIX=${R_HIGHS_LIB_DIR} -DCMAKE_POSITION_INDEPENDENT_CODE:bool=ON -DSHARED:bool=OFF -DBUILD_TESTING:bool=OFF"
+CMAKE_OPTS="-DCMAKE_INSTALL_PREFIX=${R_HIGHS_LIB_DIR} -DCMAKE_POSITION_INDEPENDENT_CODE:bool=ON -DBUILD_SHARED_LIBS:bool=OFF -DBUILD_TESTING:bool=OFF -DZLIB:bool=OFF -DBUILD_EXAMPLES:bool=OFF"
 if test "$(uname -s)" = "Darwin"; then
     # Use FastBuild only on Darwin otherwise I run in warings hell.
     ${CMAKE_EXE} .. ${CMAKE_OPTS} -DFAST_BUILD:bool=ON
 else
     # FAST_BUILD fails on Windows, for the other platforms both seams to work.
-    ${CMAKE_EXE} .. ${CMAKE_OPTS} -DFAST_BUILD:bool=OFF -G "Unix Makefiles"
+    ${CMAKE_EXE} .. ${CMAKE_OPTS} -DFAST_BUILD:bool=ON -G "Unix Makefiles"
 fi
 
 ${MAKE} install
